@@ -1,12 +1,14 @@
-#ifndef _MNTOPT_H
-#define _MNTOPT_H
+#ifndef GUARD_MNTOPT_H
+#define GUARD_MNTOPT_H
 
 #include <mntent.h>
 
 /* filesystem type */
 #define MNTTYPE_EXT2		"ext2"	/* 2nd Extended file system */
 #define MNTTYPE_EXT3		"ext3"	/* ext2 + journaling */
-#define MNTTYPE_EXT4		"ext4dev"	/* ext4 filesystem */
+#define MNTTYPE_EXT4		"ext4"	/* ext4 filesystem */
+#define MNTTYPE_NEXT3		"next3"	/* next3 filesystem */
+#define MNTTYPE_EXT4DEV		"ext4dev"/* ext4dev filesystem */
 #define MNTTYPE_MINIX		"minix"	/* MINIX file system */
 #define MNTTYPE_UFS		"ufs"	/* UNIX file system */
 #define MNTTYPE_UDF		"udf"	/* OSTA UDF file system */
@@ -16,6 +18,8 @@
 #define MNTTYPE_JFS		"jfs"	/* JFS file system */
 #define MNTTYPE_NFS4		"nfs4"	/* NFSv4 filesystem */
 #define MNTTYPE_MPFS		"mpfs"  /* EMC Celerra MPFS filesystem */
+#define MNTTYPE_OCFS2		"ocfs2"	/* Oracle Cluster filesystem */
+#define MNTTYPE_GFS2		"gfs2"	/* Red Hat Global filesystem 2 */
 
 /* mount options */
 #define MNTOPT_NOQUOTA		"noquota"	/* don't enforce quota */
