@@ -12,11 +12,13 @@ typedef int64_t qsize_t;	/* Type in which we store size limitations */
 
 /*
  * Definitions for the default names of the quotas files.
+ * N_ is gettext magic for static strings
  */
+#define N_(x) (x)
 #define INITQFNAMES { \
-	"user",    /* USRQUOTA */ \
-	"group",   /* GRPQUOTA */ \
-	"undefined", \
+	N_("user"),    /* USRQUOTA */ \
+	N_("group"),   /* GRPQUOTA */ \
+	N_("undefined"), \
 }
 
 /*
@@ -95,6 +97,8 @@ struct if_dqblk {
 #define IIF_FLAGS	4
 #define IIF_ALL		(IIF_BGRACE | IIF_IGRACE | IIF_FLAGS)
 
+#define DQF_SYS_FILE	0x10000		/* Quota stored in a system file */
+
 struct if_dqinfo {
 	u_int64_t dqi_bgrace;
 	u_int64_t dqi_igrace;
