@@ -293,7 +293,7 @@ static void report_it(struct quota_handle *h, int type)
 	else
 		spacehdr = _("Block");
 
-	printf(_("*** Report for %s quotas on device %s\n"), type2name(type), h->qh_quotadev);
+	printf(_("*** Report for %s quotas on device %s\n"), _(type2name(type)), h->qh_quotadev);
 	time2str(h->qh_info.dqi_bgrace, bgbuf, TF_ROUND);
 	time2str(h->qh_info.dqi_igrace, igbuf, TF_ROUND);
 	printf(_("Block grace time: %s; Inode grace time: %s\n"), bgbuf, igbuf);
