@@ -162,7 +162,6 @@ int rpc_rquota_get(struct dquot *dquot)
 		return -ENOENT;
 	}
 
-	*pathname++ = '\0';
 	/* For NFSv4, we send the filesystem path without initial /. Server prepends proper
 	 * NFS pseudoroot automatically and uses this for detection of NFSv4 mounts. */
 	if ((dquot->dq_h->qh_io_flags & IOFL_NFS_MIXED_PATHS) &&
@@ -265,6 +264,10 @@ int rpc_rquota_set(int qcmd, struct dquot *dquot)
 	char *fsname_tmp, *host, *pathname;
 	struct timeval timeout = { 2, 0 };
 
+	/* RPC limits values to 32b variables. Prevent value wrapping. */
+	if (check_dquot_range(dquot) < 0)
+		return -ERANGE;
+
 	/*
 	 * Convert host:pathname to seperate host and pathname.
 	 */
